@@ -8,13 +8,17 @@ import json
 import logging
 import signal
 from datetime import datetime
+from pathlib import Path
 from typing import TYPE_CHECKING, Annotated
 
 import typer
 from rich.console import Console
 
+from chzzk.cli.formatter import ChatFormatter, FormatConfig
+from chzzk.cli.logging import setup_logging
 from chzzk.cli.tui import can_run_tui, run_tui
 from chzzk.cli.tui.apps import ChatViewerApp, InteractiveChatApp
+from chzzk.cli.writers import ChatWriter, OutputFormat, create_writer
 from chzzk.constants import StatusText
 from chzzk.exceptions import ChatConnectionError, ChatNotLiveError
 from chzzk.unofficial import (
@@ -23,6 +27,7 @@ from chzzk.unofficial import (
     DonationMessage,
     UnofficialChzzkClient,
 )
+from chzzk.unofficial.models.reconnect import ReconnectEvent, ReconnectReason, StatusChangeEvent
 
 if TYPE_CHECKING:
     from chzzk.cli.config import ConfigManager
@@ -46,48 +51,52 @@ def get_auth_cookies(ctx: typer.Context) -> tuple[str | None, str | None]:
     )
 
 
-def format_chat_message(msg: ChatMessage, json_output: bool = False) -> str:
-    """Format a chat message for display."""
-    if json_output:
-        return json.dumps(
-            {
-                "type": "chat",
-                "timestamp": datetime.now().isoformat(),
-                "user_id_hash": msg.user_id_hash,
-                "nickname": msg.nickname,
-                "content": msg.content,
-                "badge": (
-                    msg.profile.badge.get("name") if msg.profile and msg.profile.badge else None
-                ),
-            }
-        )
-
-    timestamp = datetime.now().strftime("%H:%M:%S")
-    badge = ""
-    if msg.profile and msg.profile.badge:
-        badge = f"[{msg.profile.badge.get('name')}] " if msg.profile.badge.get("name") else ""
-
-    return f"[dim]{timestamp}[/dim] {badge}[cyan]{msg.nickname}[/cyan]: {msg.content}"
+def get_format_config(ctx: typer.Context) -> FormatConfig:
+    """Get FormatConfig from context."""
+    return FormatConfig.from_env_and_cli(
+        cli_chat_format=ctx.obj.get("chat_format"),
+        cli_donation_format=ctx.obj.get("donation_format"),
+        cli_sent_format=ctx.obj.get("sent_format"),
+        cli_time_format=ctx.obj.get("time_format"),
+    )
 
 
-def format_donation_message(msg: DonationMessage, json_output: bool = False) -> str:
-    """Format a donation message for display."""
-    if json_output:
-        return json.dumps(
-            {
-                "type": "donation",
-                "timestamp": datetime.now().isoformat(),
-                "user_id_hash": msg.user_id_hash,
-                "nickname": msg.nickname,
-                "content": msg.content,
-                "pay_amount": msg.pay_amount,
-            }
-        )
+def format_chat_message_json(msg: ChatMessage) -> str:
+    """Format a chat message as JSON."""
+    return json.dumps(
+        {
+            "type": "chat",
+            "timestamp": datetime.now().isoformat(),
+            "user_id_hash": msg.user_id_hash,
+            "nickname": msg.nickname,
+            "content": msg.content,
+            "badge": (msg.profile.badge.get("name") if msg.profile and msg.profile.badge else None),
+        }
+    )
 
-    timestamp = datetime.now().strftime("%H:%M:%S")
-    return (
-        f"[dim]{timestamp}[/dim] [yellow]${msg.pay_amount}[/yellow] "
-        f"[magenta]{msg.nickname}[/magenta]: {msg.content or ''}"
+
+def format_donation_message_json(msg: DonationMessage) -> str:
+    """Format a donation message as JSON."""
+    return json.dumps(
+        {
+            "type": "donation",
+            "timestamp": datetime.now().isoformat(),
+            "user_id_hash": msg.user_id_hash,
+            "nickname": msg.nickname,
+            "content": msg.content,
+            "pay_amount": msg.pay_amount,
+        }
+    )
+
+
+def format_sent_message_json(content: str) -> str:
+    """Format a sent message as JSON."""
+    return json.dumps(
+        {
+            "type": "sent",
+            "timestamp": datetime.now().isoformat(),
+            "content": content,
+        }
     )
 
 
@@ -112,6 +121,23 @@ def watch(
             help="Disable TUI and use simple console output",
         ),
     ] = False,
+    output: Annotated[
+        Path | None,
+        typer.Option(
+            "--output",
+            "-o",
+            envvar="CHZZK_CHAT_OUTPUT",
+            help="Save chat messages to file",
+        ),
+    ] = None,
+    output_format: Annotated[
+        str,
+        typer.Option(
+            "--output-format",
+            envvar="CHZZK_CHAT_OUTPUT_FORMAT",
+            help="Output format: jsonl, txt (default: jsonl)",
+        ),
+    ] = "jsonl",
 ) -> None:
     """Watch real-time chat messages from a channel.
 
@@ -120,37 +146,68 @@ def watch(
 
     By default, displays a full-screen TUI with scrollable messages.
     Use --no-tui for simple console output.
+
+    Use --output to save messages to a file while watching.
     """
     nid_aut, nid_ses = get_auth_cookies(ctx)
     json_output = ctx.obj.get("json_output", False)
     config = get_config(ctx)
+    format_config = get_format_config(ctx)
 
-    # Use TUI if available and not disabled/json mode
-    if not json_output and not no_tui and can_run_tui():
-        viewer_app = ChatViewerApp(
-            config=config,
+    # Create writer if output path specified
+    writer: ChatWriter | None = None
+    if output:
+        try:
+            writer = create_writer(output, OutputFormat(output_format))
+        except ValueError as e:
+            console.print(f"[red]Error:[/red] {e}")
+            raise typer.Exit(1) from None
+        except OSError as e:
+            console.print(f"[red]Error:[/red] Cannot write to {output}: {e}")
+            raise typer.Exit(1) from None
+
+    try:
+        # Use TUI if available and not disabled/json mode
+        if not json_output and not no_tui and can_run_tui():
+            # Reconfigure logging for TUI mode: disable console output to prevent
+            # logs from corrupting the TUI display, keep file logging if configured
+            setup_logging(
+                ctx.obj.get("log_level", "WARNING"),
+                log_file=ctx.obj.get("log_file"),
+                disable_console=True,
+            )
+
+            viewer_app = ChatViewerApp(
+                config=config,
+                channel_id=channel_id,
+                nid_aut=nid_aut,
+                nid_ses=nid_ses,
+                allow_offline=offline,
+                writer=writer,
+                format_config=format_config,
+            )
+            run_tui(viewer_app)
+
+            if viewer_app.error_message:
+                console.print(f"[red]Error:[/red] {viewer_app.error_message}")
+                raise typer.Exit(1)
+            return
+
+        # Fallback to console output
+        if not no_tui and not json_output:
+            logger.info("TUI mode unavailable, using console mode")
+        _run_watch_console(
             channel_id=channel_id,
             nid_aut=nid_aut,
             nid_ses=nid_ses,
-            allow_offline=offline,
+            offline=offline,
+            json_output=json_output,
+            writer=writer,
+            format_config=format_config,
         )
-        run_tui(viewer_app)
-
-        if viewer_app.error_message:
-            console.print(f"[red]Error:[/red] {viewer_app.error_message}")
-            raise typer.Exit(1)
-        return
-
-    # Fallback to console output
-    if not no_tui and not json_output:
-        logger.info("TUI mode unavailable, using console mode")
-    _run_watch_console(
-        channel_id=channel_id,
-        nid_aut=nid_aut,
-        nid_ses=nid_ses,
-        offline=offline,
-        json_output=json_output,
-    )
+    finally:
+        if writer:
+            writer.close()
 
 
 def _run_watch_console(
@@ -160,8 +217,11 @@ def _run_watch_console(
     nid_ses: str | None,
     offline: bool,
     json_output: bool,
+    writer: ChatWriter | None = None,
+    format_config: FormatConfig | None = None,
 ) -> None:
     """Run chat watch with console output (fallback mode)."""
+    formatter = ChatFormatter(format_config)
 
     async def run_chat() -> None:
         async with AsyncUnofficialChzzkClient(nid_aut=nid_aut, nid_ses=nid_ses) as client:
@@ -169,11 +229,83 @@ def _run_watch_console(
 
             @chat.on_chat
             async def handle_chat(msg: ChatMessage) -> None:
-                console.print(format_chat_message(msg, json_output))
+                if json_output:
+                    console.print(format_chat_message_json(msg))
+                else:
+                    console.print(formatter.format_chat(msg))
+                if writer:
+                    writer.write_chat(msg)
 
             @chat.on_donation
             async def handle_donation(msg: DonationMessage) -> None:
-                console.print(format_donation_message(msg, json_output))
+                if json_output:
+                    console.print(format_donation_message_json(msg))
+                else:
+                    console.print(formatter.format_donation(msg))
+                if writer:
+                    writer.write_donation(msg)
+
+            @chat.on_live
+            async def handle_live(event: StatusChangeEvent) -> None:
+                if json_output:
+                    console.print(
+                        json.dumps(
+                            {
+                                "event": "live",
+                                "live_id": event.live_id,
+                                "title": event.live_title,
+                            }
+                        )
+                    )
+                else:
+                    title = event.live_title or "제목 없음"
+                    console.print(f"\n[green]방송이 시작되었습니다![/green] {title}")
+
+            @chat.on_offline
+            async def handle_offline(event: StatusChangeEvent) -> None:
+                if json_output:
+                    console.print(
+                        json.dumps(
+                            {
+                                "event": "offline",
+                                "live_id": event.live_id,
+                                "title": event.live_title,
+                            }
+                        )
+                    )
+                else:
+                    console.print(
+                        "\n[yellow]방송이 종료되었습니다.[/yellow] "
+                        "재시작을 기다리는 중... (Ctrl+C로 종료)"
+                    )
+
+            @chat.on_reconnect
+            async def handle_reconnect(event: ReconnectEvent) -> None:
+                if json_output:
+                    console.print(
+                        json.dumps(
+                            {
+                                "event": "reconnect",
+                                "reason": event.reason.value,
+                                "old_chat_channel_id": event.old_chat_channel_id,
+                                "new_chat_channel_id": event.new_chat_channel_id,
+                            }
+                        )
+                    )
+                else:
+                    if event.reason == ReconnectReason.STREAM_RESTARTED:
+                        console.print("[green]채팅에 다시 연결되었습니다.[/green]\n")
+                    else:
+                        console.print(
+                            "[yellow]채팅 채널이 변경되어 다시 연결되었습니다.[/yellow]\n"
+                        )
+
+            @chat.on_reconnect_error
+            async def handle_reconnect_error(error: Exception) -> None:
+                if json_output:
+                    console.print(json.dumps({"event": "reconnect_error", "error": str(error)}))
+                else:
+                    console.print(f"[red]재연결 실패:[/red] {error}")
 
             # Get live detail first to check status
             try:
@@ -253,21 +385,6 @@ def _run_watch_console(
         asyncio.run(run_chat())
 
 
-def format_sent_message(content: str, json_output: bool = False) -> str:
-    """Format a sent message for display."""
-    if json_output:
-        return json.dumps(
-            {
-                "type": "sent",
-                "timestamp": datetime.now().isoformat(),
-                "content": content,
-            }
-        )
-
-    timestamp = datetime.now().strftime("%H:%M:%S")
-    return f"[dim]{timestamp}[/dim] [green bold]>[/green bold] [green]{content}[/green]"
-
-
 @app.command()
 def send(
     ctx: typer.Context,
@@ -301,6 +418,23 @@ def send(
             help="Disable TUI and use simple console input/output",
         ),
     ] = False,
+    output: Annotated[
+        Path | None,
+        typer.Option(
+            "--output",
+            "-o",
+            envvar="CHZZK_CHAT_OUTPUT",
+            help="Save chat messages to file (interactive mode only)",
+        ),
+    ] = None,
+    output_format: Annotated[
+        str,
+        typer.Option(
+            "--output-format",
+            envvar="CHZZK_CHAT_OUTPUT_FORMAT",
+            help="Output format: jsonl, txt (default: jsonl)",
+        ),
+    ] = "jsonl",
 ) -> None:
     """Send a chat message to a channel.
 
@@ -312,10 +446,13 @@ def send(
 
     In interactive mode, displays a full-screen TUI by default.
     Use --no-tui for simple console input/output.
+
+    Use --output to save messages to a file (interactive mode only).
     """
     nid_aut, nid_ses = get_auth_cookies(ctx)
     json_output = ctx.obj.get("json_output", False)
     config = get_config(ctx)
+    format_config = get_format_config(ctx)
 
     if not nid_aut or not nid_ses:
         if json_output:
@@ -338,45 +475,80 @@ def send(
             )
         raise typer.Exit(1)
 
-    if interactive:
-        # Use TUI if available and not disabled/json mode
-        if not json_output and not no_tui and can_run_tui():
-            interactive_app = InteractiveChatApp(
+    # Warn if output is specified in non-interactive mode
+    if output and not interactive:
+        if not json_output:
+            console.print("[yellow]Warning:[/yellow] --output is ignored in non-interactive mode.")
+        output = None
+
+    # Create writer if output path specified (interactive mode only)
+    writer: ChatWriter | None = None
+    if output:
+        try:
+            writer = create_writer(output, OutputFormat(output_format))
+        except ValueError as e:
+            console.print(f"[red]Error:[/red] {e}")
+            raise typer.Exit(1) from None
+        except OSError as e:
+            console.print(f"[red]Error:[/red] Cannot write to {output}: {e}")
+            raise typer.Exit(1) from None
+
+    try:
+        if interactive:
+            # Use TUI if available and not disabled/json mode
+            if not json_output and not no_tui and can_run_tui():
+                # Reconfigure logging for TUI mode: disable console output to prevent
+                # logs from corrupting the TUI display, keep file logging if configured
+                setup_logging(
+                    ctx.obj.get("log_level", "WARNING"),
+                    log_file=ctx.obj.get("log_file"),
+                    disable_console=True,
+                )
+
+                interactive_app = InteractiveChatApp(
+                    config=config,
+                    channel_id=channel_id,
+                    nid_aut=nid_aut,
+                    nid_ses=nid_ses,
+                    allow_offline=offline,
+                    writer=writer,
+                    format_config=format_config,
+                )
+                run_tui(interactive_app)
+
+                if interactive_app.error_message:
+                    console.print(f"[red]Error:[/red] {interactive_app.error_message}")
+                    raise typer.Exit(1)
+                return
+
+            # Fallback to console interactive mode
+            if not no_tui and not json_output:
+                logger.info("TUI mode unavailable, using console mode")
+            _run_interactive_chat_console(
+                ctx=ctx,
                 config=config,
                 channel_id=channel_id,
                 nid_aut=nid_aut,
                 nid_ses=nid_ses,
-                allow_offline=offline,
+                offline=offline,
+                json_output=json_output,
+                writer=writer,
+                format_config=format_config,
             )
-            run_tui(interactive_app)
-
-            if interactive_app.error_message:
-                console.print(f"[red]Error:[/red] {interactive_app.error_message}")
-                raise typer.Exit(1)
-            return
-
-        # Fallback to console interactive mode
-        if not no_tui and not json_output:
-            logger.info("TUI mode unavailable, using console mode")
-        _run_interactive_chat_console(
-            config=config,
-            channel_id=channel_id,
-            nid_aut=nid_aut,
-            nid_ses=nid_ses,
-            offline=offline,
-            json_output=json_output,
-        )
-    else:
-        # message is guaranteed to be non-None here (checked above)
-        assert message is not None
-        _send_single_message(
-            channel_id=channel_id,
-            message=message,
-            nid_aut=nid_aut,
-            nid_ses=nid_ses,
-            offline=offline,
-            json_output=json_output,
-        )
+        else:
+            # message is guaranteed to be non-None here (checked above)
+            assert message is not None
+            _send_single_message(
+                channel_id=channel_id,
+                message=message,
+                nid_aut=nid_aut,
+                nid_ses=nid_ses,
+                offline=offline,
+                json_output=json_output,
+            )
+    finally:
+        if writer:
+            writer.close()
 
 
 def _send_single_message(
@@ -438,12 +610,15 @@ def _send_single_message(
 
 def _run_interactive_chat_console(
     *,
+    ctx: typer.Context,
     config: ConfigManager,
     channel_id: str,
     nid_aut: str,
     nid_ses: str,
     offline: bool,
     json_output: bool,
+    writer: ChatWriter | None = None,
+    format_config: FormatConfig | None = None,
 ) -> None:
     """Run interactive chat with console input/output (fallback mode).
 
@@ -451,6 +626,14 @@ def _run_interactive_chat_console(
     For JSON mode, uses async stdin reading with timeout on cancellation.
     """
     if not json_output:
+        # Reconfigure logging for inline TUI mode: disable console output to prevent
+        # logs from corrupting the TUI display, keep file logging if configured
+        setup_logging(
+            ctx.obj.get("log_level", "WARNING"),
+            log_file=ctx.obj.get("log_file"),
+            disable_console=True,
+        )
+
         # Use inline Textual app for non-JSON mode - this allows proper cancellation
         interactive_app = InteractiveChatApp(
             config=config,
@@ -459,6 +642,8 @@ def _run_interactive_chat_console(
             nid_ses=nid_ses,
             allow_offline=offline,
             inline_mode=True,
+            writer=writer,
+            format_config=format_config,
         )
         run_tui(interactive_app, inline=True, inline_height=15)
 
@@ -473,6 +658,7 @@ def _run_interactive_chat_console(
         nid_aut=nid_aut,
         nid_ses=nid_ses,
         offline=offline,
+        writer=writer,
     )
 
 
@@ -482,6 +668,7 @@ def _run_interactive_chat_json(
     nid_aut: str,
     nid_ses: str,
     offline: bool,
+    writer: ChatWriter | None = None,
 ) -> None:
     """Run interactive chat with JSON output (no TUI).
 
@@ -494,11 +681,56 @@ def _run_interactive_chat_json(
 
             @chat.on_chat
             async def handle_chat(msg: ChatMessage) -> None:
-                console.print(format_chat_message(msg, json_output=True))
+                console.print(format_chat_message_json(msg))
+                if writer:
+                    writer.write_chat(msg)
 
             @chat.on_donation
             async def handle_donation(msg: DonationMessage) -> None:
-                console.print(format_donation_message(msg, json_output=True))
+                console.print(format_donation_message_json(msg))
+                if writer:
+                    writer.write_donation(msg)
+
+            @chat.on_live
+            async def handle_live(event: StatusChangeEvent) -> None:
+                console.print(
+                    json.dumps(
+                        {
+                            "event": "live",
+                            "live_id": event.live_id,
+                            "title": event.live_title,
+                        }
+                    )
+                )
+
+            @chat.on_offline
+            async def handle_offline(event: StatusChangeEvent) -> None:
+                console.print(
+                    json.dumps(
+                        {
+                            "event": "offline",
+                            "live_id": event.live_id,
+                            "title": event.live_title,
+                        }
+                    )
+                )
+
+            @chat.on_reconnect
+            async def handle_reconnect(event: ReconnectEvent) -> None:
+                console.print(
+                    json.dumps(
+                        {
+                            "event": "reconnect",
+                            "reason": event.reason.value,
+                            "old_chat_channel_id": event.old_chat_channel_id,
+                            "new_chat_channel_id": event.new_chat_channel_id,
+                        }
+                    )
+                )
+
+            @chat.on_reconnect_error
+            async def handle_reconnect_error(error: Exception) -> None:
+                console.print(json.dumps({"event": "reconnect_error", "error": str(error)}))
 
             # Get live detail first to check status (validates channel exists)
             try:
@@ -545,7 +777,9 @@ def _run_interactive_chat_json(
                         line = await asyncio.to_thread(input)
                         if line and not stop_event.is_set():
                             await chat.send_message(line)
-                            console.print(format_sent_message(line, json_output=True))
+                            console.print(format_sent_message_json(line))
+                            if writer:
+                                writer.write_sent(line)
                     except EOFError:
                         # stdin closed
                         break
